{% extends "base.html" %}
{% import "macros.html" as macros %}

{% block title %}Thread Detail Renderer{% endblock %}

{% block content %}
    {# Render the main thread #}
    {{ macros.content_item(thread, is_thread=True, forum_name=forum, forum_icon_url=forum_icon_url) }}

    {# Render posts #}
    {% if posts %}
    <div class="posts">
        {% for post in posts %}
            {{ macros.content_item(post) }}
        {% endfor %}
    </div>
    {% endif %}
{% endblock %}
